#include <stdio.h>

int main(int argc,char *argv[]) {
	FILE *fp1,*fp2;
	size_t sz;
	char file_header[7];
	char random_data[16];
	int block_size;
	int signature_size;
	char block_flag;
	char block_data[0x10000];
	char signature_data[0x80];
	int bn,rc,bs,bf,i;
	char nn[32768];

	if(argc<2) {
		printf("usage: nbh2nbg <NBH file>\n");
		return(1);
	}

	fp1=fopen(argv[1],"rb");
	if(fp1==NULL) {
		printf("NBH file open error.\n");
		return(1);
	}

	sz=fread(file_header,sizeof(file_header),1,fp1);
	if(sz<1) {
		printf("file header read error.\n");
		fclose(fp1);
		return(1);
	}
	if(file_header[6]!=0x0a) {
		printf("file header error.\n");
		fclose(fp1);
		return(1);
	}
	file_header[6]='\0';
	if(strcmp(file_header,"R000FF")!=0) {
		printf("file header error.\n");
		fclose(fp1);
		return(1);
	}

	sz=fread(random_data,sizeof(random_data),1,fp1);
	if(sz<1) {
		printf("random data read error.\n");
		fclose(fp1);
		return(1);

	}

	strcpy(nn,argv[1]);
	i=strlen(nn);
	while(i>0 && nn[i]!='.') i--;
	if(nn[i]!='.') i=strlen(nn);
	if(i>32763) {
		printf("NBH file full path is longer.\n");
		fclose(fp1);
		return(1);
	}
	nn[i]='.';
	nn[i+1]='n';
	nn[i+2]='b';
	nn[i+3]='g';
	nn[i+4]='\0';

	fp2=fopen(nn,"wb");
	if(fp2==NULL) {
		printf("NBG file open error.\n");
		return(1);
	}

	rc=0;
	bn=0;
	bs=0x10000;
	bf=0;
	while(1) {

		sz=fread(&block_size,sizeof(block_size),1,fp1);
		if(sz<1) {
			printf("block %d : block size read error.\n",bn);
			rc=1;
			break;
		}
		if(block_size>0x10000) {
			printf("block %d : block size error. (block size = 0x%X)\n",bn,block_size);
			rc=1;
			break;
		}
		if(block_size!=0 && block_size>bs) {
			printf("block %d : block size error. (before block size = 0x%X , block size = 0x%X)\n",bn,bf,block_size);
			rc=1;
			break;
		}

		sz=fread(&signature_size,sizeof(signature_size),1,fp1);
		if(sz<1) {
			printf("block %d : signature size read error.\n",bn);
			rc=1;
			break;
		}
		if(signature_size>0x80) {
			printf("block %d : signature size error. (signature size = 0x%X)\n",bn,signature_size);
			rc=1;
			break;
		}
		if(signature_size<0x80) {
			printf("block %d : signature size warning. (signature size = 0x%X)\n",bn,signature_size);
		}

		sz=fread(&block_flag,sizeof(block_flag),1,fp1);
		if(sz<1) {
			printf("block %d : block flag read error.\n",bn);
			rc=1;
			break;
		}
		if(block_flag<1) {
			printf("block %d : block flag warning. (block flag = %d)\n",bn,block_flag);
		}
		if(block_flag>2) {
			printf("block %d : block flag warning. (block flag = %d)\n",bn,block_flag);
		}
		if(block_size>0 && block_flag!=1) {
			printf("block %d : block flag warning. (block size = 0x%X , block flag = %d)\n",bn,block_size,block_flag);
		}
		if(block_size==0 && block_flag!=2) {
			printf("block %d : block flag warning. (block size = 0x%X , block flag = %d)\n",bn,block_size,block_flag);
		}
		if(block_flag<bf) {
			printf("block %d : block flag warning. (before block flag = %d , block flag = %d)\n",bn,bf,block_flag);
		}

		if(block_size>0) {
			sz=fread(block_data,1,block_size,fp1);
			if(sz<block_size) {
				printf("block %d : block data read error.\n",bn);
				rc=1;
				break;
			}
			sz=fwrite(block_data,1,block_size,fp2);
			if(sz<block_size) {
				printf("block %d : block data write error.\n",bn);
				rc=1;
				break;
			}
		}

		sz=fread(signature_data,1,signature_size,fp1);
		if(sz<signature_size) {
			printf("block %d : signature data read error.\n",bn);
			rc=1;
			break;
		}

		bs=block_size;
		bf=block_flag;
		bn++;

		if(block_size==0) {
			printf("%d block finished.\n",bn);
			rc=0;
			break;
		}
	}
	fclose(fp2);
	fclose(fp1);
	return(rc);
}
